use color_eyre::{eyre::ContextCompat, Result};
use tiny_skia::{ColorU8, Pixmap, PremultipliedColorU8};

use crate::model::image_filter::ImageFilter;

pub fn apply(original: &Pixmap, filter: &ImageFilter) -> Result<Pixmap> {
    let mut result = if let Some(rect) = filter.crop {
        original.clone_rect(*rect).wrap_err_with(|| format!("Invalid crop rect: {}", rect))?
    } else {
        original.clone()
    };

    // scale, rotate and border are handled in runner::graphics::render_key

    for p in result.pixels_mut() {
        if filter.invert {
            *p = PremultipliedColorU8::from_rgba(p.alpha() - p.red(), p.alpha() - p.green(), p.alpha() - p.blue(), p.alpha()).unwrap();
        }

        if filter.alpha != 1.0 {
            let d = p.demultiply();
            *p = ColorU8::from_rgba(d.red(), d.green(), d.blue(), (d.alpha() as f32 * filter.alpha).round() as u8).premultiply();
        }

        if filter.grayscale {
            let a = p.alpha();
            if a > 0 {
                // The values are adjusted to make it look right on the Loupedeck Live display
                let luminosity = (0.3 * p.red() as f32 + 0.6 * p.green() as f32 + 0.1 * p.blue() as f32).floor() as u8;
                *p = ColorU8::from_rgba(luminosity, luminosity, luminosity, a).premultiply();
            }
        }

        if let Some(color) = &filter.color {
            let a = p.alpha();
            if a > 0 {
                *p = ColorU8::from_rgba(color.r, color.g, color.b, a).premultiply();
            }
        }
    }

    Ok(result)
}
